@@ -5,7 +5,7 @@ comp = pnl.Composition(name="comp")
 fn = pnl.IntegratorMechanism(
     name="fn",
     function=pnl.FitzHughNagumoIntegrator(
-        name="FitzHughNagumoIntegrator Function-0",
+        name="FitzHughNagumoIntegrator_Function_0",
         d_v=1,
         initial_v=-1,
         initializer=[[0]],
@@ -19,6 +19,6 @@ comp.add_node(fn)
 comp.scheduler.add_condition(fn, pnl.Always())
 
 comp.scheduler.termination_conds = {
-    pnl.TimeScale.RUN: pnl.Never(),
-    pnl.TimeScale.TRIAL: pnl.AllHaveRun(),
+    pnl.TimeScale.ENVIRONMENT_SEQUENCE: pnl.Never(),
+    pnl.TimeScale.ENVIRONMENT_STATE_UPDATE: pnl.AllHaveRun(),
 }
