@@ -11,6 +11,7 @@ of the :class:`EvaluableGraph` class. The external library `graph-scheduler
 conditional constraints.
 
 """
+import functools
 import inspect
 import os
 import re
@@ -20,13 +21,14 @@ import numpy as np
 
 
 import graph_scheduler
+import onnxruntime
 
 from modeci_mdf.functions.standard import mdf_functions, create_python_expression
 from modeci_mdf.utils import is_number
 
-from neuromllite.utils import evaluate as evaluate_params_nmllite
-from neuromllite.utils import _params_info, _val_info
-from neuromllite.utils import FORMAT_NUMPY
+from modelspec.utils import evaluate as evaluate_params_modelspec
+from modelspec.utils import _params_info, _val_info
+from modelspec.utils import FORMAT_NUMPY
 
 from collections import OrderedDict
 from typing import Union, List, Dict, Optional, Any
@@ -70,7 +72,7 @@ def evaluate_expr(
 
     """
 
-    e = evaluate_params_nmllite(
+    e = evaluate_params_modelspec(
         expr, func_params, array_format=array_format, verbose=verbose
     )
     if type(e) == str and e not in KNOWN_PARAMETERS:
@@ -81,6 +83,190 @@ def evaluate_expr(
     return e
 
 
+def evaluate_onnx_expr(
+    expr: str,
+    base_parameters: Dict[str, Any],
+    evaluated_parameters: Dict,
+    verbose: bool = False,
+) -> Any:
+    """Evaluates a simple expression in string format representing an
+    ONNX function call
+
+    Args:
+        expr (str): Expression to be evaluated
+        base_parameters (Dict[str, Any]): A dict of parameters that may contain variables
+        evaluated_parameters (Dict): A dict mapping variables used in **base_parameters** to actual values
+        verbose (bool, optional): If set to True provides in-depth information else verbose message is not displayed. Defaults to False.
+
+    Returns:
+        Any: the return value of **expr**
+    """
+    # Get the ONNX function
+    onnx_name = expr.split("(")[0].split(".")[-1]
+    onnx_function = getattr(onnx_ops, onnx_name)
+    onnx_schema = onnx_ops.get_onnx_schema(onnx_name)
+    onnx_arguments = set(
+        list(onnx_schema.attributes.keys()) + [i.name for i in onnx_schema.inputs]
+    )
+    # used to attempt to match inputs to expected onnx input types
+    onnx_typecast_mappings = {
+        onnx_schema.AttrType.INT: int,
+        onnx_schema.AttrType.FLOAT: float,
+        onnx_schema.AttrType.STRING: str,
+        onnx_schema.AttrType.INTS: functools.partial(np.array, dtype=int),
+        onnx_schema.AttrType.FLOATS: functools.partial(np.array, dtype=float),
+        onnx_schema.AttrType.STRINGS: functools.partial(np.array, dtype=str),
+        # TODO: add tensor and graph types?
+    }
+
+    try:
+        has_variadic = (
+            onnx_schema.inputs[0].option == onnx_schema.FormalParameterOption.Variadic
+        )
+    except IndexError:
+        has_variadic = False
+
+    # ONNX functions expect input args or kwargs first, followed by parameters (called attributes in ONNX) as
+    # kwargs. Lets construct this.
+    kwargs_for_onnx = {}
+    for kw, arg_expr in base_parameters.items():
+
+        # If this arg is a list of args, we are dealing with a variadic argument. Expand these
+        if type(arg_expr) == str and arg_expr[0] == "[" and arg_expr[-1] == "]":
+            # Use the Python interpreter to parse this into a List[str]
+            arg_expr_list = parse_str_as_list(arg_expr[1:-1])
+            kwargs_for_onnx.update({a: evaluated_parameters[a] for a in arg_expr_list})
+        else:
+            kwargs_for_onnx[kw] = evaluated_parameters[kw]
+
+    kwargs_for_onnx = {
+        k: v
+        for k, v in kwargs_for_onnx.items()
+        if (
+            (k in onnx_arguments or has_variadic)
+            and "onnx::" not in k  # filter Evaluable__ class names
+        )
+    }
+
+    # attempt to cast attributes to what onnx_function expects
+    for k, v in kwargs_for_onnx.items():
+        try:
+            onnx_attr = onnx_schema.attributes[k]
+        except KeyError:
+            continue
+
+        try:
+            cast_type = onnx_typecast_mappings[onnx_attr.type]
+        except KeyError:
+            continue
+
+        try:
+            kwargs_for_onnx[k] = cast_type(v)
+        except (TypeError, ValueError):
+            pass
+
+    if verbose:
+        print(f"Evaluating ONNX function {onnx_name} with {kwargs_for_onnx}")
+
+    try:
+        result = onnx_function(**kwargs_for_onnx)
+    except (
+        onnxruntime.capi.onnxruntime_pybind11_state.NotImplemented,
+        onnxruntime.capi.onnxruntime_pybind11_state.Fail,
+    ) as e:
+        err = str(e)
+        if (
+            "bound to different types (tensor(double) and tensor(float)" not in err
+            and "Could not find an implementation for the node" not in err
+        ):
+            raise
+
+        # assume this is related to lack of support for float64/double
+        # for Cos, Relu (and likely others) on onnx CPUExecutionProvider
+        result = onnx_function(
+            **{
+                k: v.astype(np.float32)
+                if hasattr(v, "dtype") and v.dtype == np.float64
+                else v
+                for k, v in kwargs_for_onnx.items()
+            }
+        )
+
+    try:
+        if result.dtype == np.float32:
+            result = result.astype(np.float64)
+    except AttributeError:
+        pass
+
+    return result
+
+
+def parse_str_as_list(s: str) -> list:
+    """Produces a list from its string representation. Runs `eval` on
+    non-list elements within
+
+    Args:
+        s (str): a string representing a list
+
+    Returns:
+        list: a list containing elements from **s**
+    """
+
+    def try_eval_str(t):
+        try:
+            return eval(t)
+        except (NameError, SyntaxError):
+            return t
+
+    def _parse_str_as_list(t):
+        res = []
+        i = 0
+        j = 0
+        depth = 0
+
+        t = t.replace(" ", "")
+
+        while j < len(t):
+            if t[j] == "[":
+                depth += 1
+
+            if t[j] == "]":
+                depth -= 1
+
+                if depth == 0:
+                    res.append(_parse_str_as_list(t[i + 1 : j]))
+                    # also passes this check if at end of string
+                    if t[j : j + 1] not in ",]":
+                        raise ValueError(f"Malformed input at index {j} of {t} {t[j]}")
+                    i = j + 1
+
+            if t[j] == ",":
+                if depth == 0:
+                    if j > i:
+                        res.append(try_eval_str(t[i:j]))
+                    i = j + 1
+
+            j = j + 1
+
+        if depth > 0:
+            raise ValueError(f"Unmatched opening bracket in {t}")
+        elif depth < 0:
+            raise ValueError(f"Unmatched closing bracket in {t}")
+
+        if j > i:
+            res.append(try_eval_str(t[i:j]))
+
+        return res
+
+    res = _parse_str_as_list(s)
+
+    # avoid adding extra unnecessary list
+    if len(res) == 1:
+        return res[0]
+    else:
+        return res
+
+
 class EvaluableFunction:
     """Evaluates a :class:`~modeci_mdf.mdf.Function` value during MDF graph execution.
 
@@ -112,20 +298,28 @@ class EvaluableFunction:
 
         expr = None
 
+        # print("functions value and function>>>", self.function.value, self.function.function)
+
+        # func_val  = self.function.value
+
         if self.function.function:
 
             for f in mdf_functions:
-                if self.function.function == f:
+
+                if f in self.function.function.keys():
+
                     expr = create_python_expression(
                         mdf_functions[f]["expression_string"]
                     )
 
-        else:
+                    break
+
+        if expr is None:
             expr = self.function.value
-            # raise "Unknown function: {}. Known functions: {}".format(
-            #    self.function.function,
-            #    mdf_functions.keys,
-            # )
+        #     #raise "Unknown function: {}. Known functions: {}".format(
+        #     #    self.function.function,
+        #     #    mdf_functions.keys,
+        #     #)
 
         func_params = {}
         func_params.update(parameters)
@@ -134,47 +328,40 @@ class EvaluableFunction:
                 "    Evaluating %s with %s, i.e. [%s]"
                 % (self.function, _params_info(func_params), expr)
             )
-        if self.function.args:
+        if self.function.function:
 
-            for arg in self.function.args:
-                func_params[arg] = evaluate_expr(
-                    self.function.args[arg],
-                    func_params,
-                    verbose=False,
-                    array_format=array_format,
-                )
-                if self.verbose:
-                    print(
-                        "      Arg: {} became: {}".format(
-                            arg, _val_info(func_params[arg])
+            for f in mdf_functions:
+
+                if f in self.function.function.keys():
+
+                    for arg in self.function.function[f].keys():
+
+                        func_params[arg] = evaluate_expr(
+                            self.function.function[f][arg],
+                            func_params,
+                            verbose=False,
+                            array_format=array_format,
                         )
-                    )
+                        if self.verbose:
+                            print(
+                                "      Arg: {} became: {}".format(
+                                    arg, _val_info(func_params[arg])
+                                )
+                            )
+                    break
 
-        # If this is an ONNX operation, evaluate it without neuromlite.
+        # If this is an ONNX operation, evaluate it without modelspec.
 
         if "onnx_ops." in expr:
-            # Get the ONNX function
-            onnx_function = getattr(onnx_ops, expr.split("(")[0].split(".")[-1])
-
-            # ONNX functions expect input args or kwargs first, followed by parameters (called attributes in ONNX) as
-            # kwargs. Lets construct this.
-            kwargs_for_onnx = {}
-            for kw, arg_expr in self.function.args.items():
-
-                # If this arg is a list of args, we are dealing with a variadic argument. Expand these
-                if type(arg_expr) == str and arg_expr[0] == "[" and arg_expr[-1] == "]":
-                    # Use the Python interpreter to parse this into a List[str]
-                    arg_expr_list = eval(arg_expr)
-                    kwargs_for_onnx.update({a: func_params[a] for a in arg_expr_list})
-                else:
-                    kwargs_for_onnx[kw] = func_params[kw]
-
-            # Now add anything in parameters that isn't already specified as an input argument
-            for kw, arg in parameters.items():
-                if kw not in self.function.args.values():
-                    kwargs_for_onnx[kw] = arg
-
-            self.curr_value = onnx_function(**kwargs_for_onnx)
+            if self.verbose:
+                print(f"{self.function.id} is evaluating ONNX function {expr}")
+            self.curr_value = evaluate_onnx_expr(
+                expr,
+                # parameters get overridden by self.function.args
+                {**parameters, **self.function.args},
+                func_params,
+                self.verbose,
+            )
         elif "actr." in expr:
             actr_function = getattr(actr_funcs, expr.split("(")[0].split(".")[-1])
             self.curr_value = actr_function(
@@ -214,9 +401,14 @@ class EvaluableParameter:
                 self.curr_value = float(self.parameter.default_initial_value)
 
             else:
-
-                self.curr_value = self.parameter.default_initial_value
-
+                try:
+                    self.curr_value = evaluate_expr(
+                        self.parameter.default_initial_value,
+                        None,
+                    )
+                # evaluate_expr raises Exception
+                except Exception:
+                    self.curr_value = self.parameter.default_initial_value
         else:
             self.curr_value = None
 
@@ -334,46 +526,17 @@ class EvaluableParameter:
                         )
                     )
 
-            # If this is an ONNX operation, evaluate it without neuromlite.
+            # If this is an ONNX operation, evaluate it without modelspec.
             if "onnx_ops." in expr:
-                # Get the ONNX function
-                onnx_function = getattr(onnx_ops, expr.split("(")[0].split(".")[-1])
-
-                # ONNX functions expect input args or kwargs first, followed by parameters (called attributes in ONNX) as
-                # kwargs. Lets construct this.
-                kwargs_for_onnx = {}
-                for kw, arg_expr in self.parameter.args.items():
-
-                    # If this arg is a list of args, we are dealing with a variadic argument. Expand these
-                    if (
-                        type(arg_expr) == str
-                        and arg_expr[0] == "["
-                        and arg_expr[-1] == "]"
-                    ):
-                        # Use the Python interpreter to parse this into a List[str]
-                        arg_expr_list = eval(arg_expr)
-                        kwargs_for_onnx.update(
-                            {a: func_params[a] for a in arg_expr_list}
-                        )
-                    else:
-                        kwargs_for_onnx[kw] = func_params[kw]
-
-                # Now add anything in parameters that isn't already specified as an input argument
-                for kw, arg in parameters.items():
-
-                    if (
-                        kw not in self.parameter.args.values()
-                        and kw != self.parameter.id
-                        and kw != "__builtins__"
-                    ):
-                        kwargs_for_onnx[kw] = arg
                 if self.verbose:
-                    print(
-                        "%s is evaluating ONNX function %s with %s"
-                        % (self.parameter.id, expr, kwargs_for_onnx)
-                    )
-                self.curr_value = onnx_function(**kwargs_for_onnx)
-
+                    print(f"{self.parameter.id} is evaluating ONNX function {expr}")
+                self.curr_value = evaluate_onnx_expr(
+                    expr,
+                    # parameters get overridden by self.parameter.args
+                    {**parameters, **self.parameter.args},
+                    func_params,
+                    self.verbose,
+                )
             elif "actr." in expr:
                 actr_function = getattr(actr_funcs, expr.split("(")[0].split(".")[-1])
                 self.curr_value = actr_function(
@@ -573,7 +736,7 @@ class EvaluableNode:
                         and arg_expr[-1] == "]"
                     ):
                         # Use the Python interpreter to parse this into a List[str]
-                        arg_expr_list = eval(arg_expr)
+                        arg_expr_list = parse_str_as_list(arg_expr)
                     else:
                         arg_expr_list = [arg_expr]
 
@@ -619,8 +782,15 @@ class EvaluableNode:
             all_req_vars = []
 
             if p.value is not None and type(p.value) == str:
-                param_expr = sympy.simplify(p.value)
-                all_req_vars.extend([str(s) for s in param_expr.free_symbols])
+                if p.value[0] == "[" and p.value[-1] == "]":
+                    # Use the Python interpreter to parse this into a List[str]
+                    arg_expr_list = parse_str_as_list(p.value)
+                else:
+                    arg_expr_list = [p.value]
+
+                for e in arg_expr_list:
+                    param_expr = sympy.simplify(e)
+                    all_req_vars.extend([str(s) for s in param_expr.free_symbols])
 
             if p.args is not None:
                 for arg in p.args:
@@ -633,7 +803,7 @@ class EvaluableNode:
                         and arg_expr[-1] == "]"
                     ):
                         # Use the Python interpreter to parse this into a List[str]
-                        arg_expr_list = eval(arg_expr)
+                        arg_expr_list = parse_str_as_list(arg_expr)
                     else:
                         arg_expr_list = [arg_expr]
 
@@ -894,7 +1064,9 @@ class EvaluableGraph:
         input_value = value if weight == 1 else value * weight
         post_node.evaluable_inputs[edge.receiver_port].set_input_value(input_value)
 
-    def parse_condition(self, condition: Condition) -> graph_scheduler.Condition:
+    def parse_condition(
+        self, condition: Union[Condition, dict]
+    ) -> graph_scheduler.Condition:
         """Convert the condition in a specific format
 
         Args:
@@ -904,15 +1076,35 @@ class EvaluableGraph:
             Condition in specific format
 
         """
+
+        def substitute_condition_arguments(condition, condition_type):
+            # mdf format prefers the key name for all conditions that use it or
+            # its value as an __init__ argument
+            combined_condition_arguments = {"dependencies": "dependency"}
+            sig = inspect.signature(condition_type)
+
+            for preferred, actual in combined_condition_arguments.items():
+                if (
+                    preferred in condition.args
+                    and preferred not in sig.parameters
+                    and actual in sig.parameters
+                ):
+                    condition.args[actual] = condition.args[preferred]
+                    del condition.args[preferred]
+
+        # if specified as dict
         try:
-            cond_type = condition["type"]
-        except TypeError:
-            cond_type = condition.type
+            args = condition["args"]
+        except (TypeError, KeyError):
+            args = {}
 
         try:
-            cond_args = condition["args"]
-        except TypeError:
-            cond_args = condition.args
+            condition = Condition(condition["type"], **args)
+        except (TypeError, KeyError):
+            pass
+
+        cond_type = condition.type
+        cond_args = condition.args
 
         try:
             typ = getattr(graph_scheduler.condition, cond_type)
@@ -946,6 +1138,8 @@ class EvaluableGraph:
             else:
                 cond_args[k] = new_v
 
+        substitute_condition_arguments(condition, typ)
+
         try:
             return typ(**cond_args)
         except TypeError as e:
@@ -978,7 +1172,7 @@ class EvaluableGraph:
                     )
 
 
-from neuromllite.utils import FORMAT_NUMPY
+from modelspec.utils import FORMAT_NUMPY
 
 
 def main(example_file: str, array_format: str = FORMAT_NUMPY, verbose: bool = False):
@@ -1021,7 +1215,7 @@ if __name__ == "__main__":
     else:
         verbose = False
 
-    from neuromllite.utils import FORMAT_NUMPY, FORMAT_TENSORFLOW
+    from modelspec.utils import FORMAT_NUMPY, FORMAT_TENSORFLOW
 
     format = FORMAT_TENSORFLOW if "-tf" in sys.argv else FORMAT_NUMPY
 
